#include "MozillaPrivate.h"

#include "galeon-embed.h"

#include <glib/gi18n.h>

#include <nsIPrefService.h>
#include <nsIPrintSettingsService.h>
#include <nsIServiceManagerUtils.h>
#include <nsIWindowWatcher.h>
#include <nsIEmbeddingSiteWindow.h>
#include <nsIWebBrowserChrome.h>
#include <nsString.h>
#include <nsPrintfCString.h>
#include <gtkmozembed.h>

GtkWidget *MozillaFindEmbed (nsIDOMWindow *aDOMWindow)
{
        nsresult result;

        nsCOMPtr<nsIWindowWatcher> wwatch
                (do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
        if (!wwatch) return nsnull;

	nsCOMPtr<nsIDOMWindow> domWindow;
	if (aDOMWindow)
	{
		/* Get the top-level Window - bug 130425 */
		aDOMWindow->GetTop (getter_AddRefs (domWindow));
	}
	else
	{
		wwatch->GetActiveWindow(getter_AddRefs(domWindow));
	}
	NS_ENSURE_TRUE (domWindow, nsnull);

        nsCOMPtr<nsIWebBrowserChrome> windowChrome;
        result = wwatch->GetChromeForWindow(domWindow,
                                            getter_AddRefs(windowChrome));
        if (NS_FAILED(result)) return nsnull;

        nsCOMPtr<nsIEmbeddingSiteWindow> window
                        (do_QueryInterface(windowChrome, &result));
        if (NS_FAILED(result)) return nsnull;

        GtkWidget *mozembed;
        result = window->GetSiteWindow((void **)&mozembed);
        if (NS_FAILED(result)) return nsnull;

	return GTK_WIDGET(mozembed);
}

GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow)
{
        GtkWidget *embed = MozillaFindEmbed (aDOMWindow);
        if (!embed) return nsnull;

        return gtk_widget_get_toplevel (GTK_WIDGET (embed));
}

NS_METHOD MozillaPrintSettingsToEmbedPrintInfo(nsIPrintSettings *aPrintSettings,
                                               EmbedPrintInfo **aPrintInfo)
{
	nsXPIDLString tmpString;
	nsresult rv = NS_OK;
	EmbedPrintInfo *info = g_new0(EmbedPrintInfo, 1);

	info->paper_array = g_ptr_array_new();
	info->paper_desc_array = g_ptr_array_new();

	aPrintSettings->GetPrinterName(getter_Copies(tmpString));
	info->name = g_strdup(NS_ConvertUCS2toUTF8(tmpString).get());
	aPrintSettings->GetPrintCommand(getter_Copies(tmpString));
	info->command = g_strdup(NS_ConvertUCS2toUTF8(tmpString).get());

	aPrintSettings->GetPrintToFile(&info->print_to_file);

	nsCAutoString printPrefix("print.tmp.printerfeatures.");
	printPrefix.Append(info->name);

	nsCOMPtr<nsIPrefService> prefService =
				do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
	NS_ENSURE_SUCCESS(rv, rv);

	nsCOMPtr<nsIPrefBranch> pref;
	rv = prefService->GetBranch(printPrefix.get(),
				    getter_AddRefs(pref));
	NS_ENSURE_SUCCESS(rv, rv);

        rv = pref->GetBoolPref(".can_change_spoolercommand",
                               &info->can_change_command);
	NS_ENSURE_SUCCESS(rv, rv);

	gint numPapers;
	rv = pref->GetIntPref(".paper.count", &numPapers);
	for (int i = 0; i < numPapers; i++)
	{
		nsPrintfCString paperPrefix(".paper.%d.", i);
                nsCAutoString tmp;

		nsXPIDLCString name;
		tmp = paperPrefix + NS_LITERAL_CSTRING("name");
		rv = pref->GetCharPref(tmp.get(), getter_Copies(name));

		g_ptr_array_add(info->paper_array, g_strdup(name.get()));

		PRInt32 height, width;
		tmp = paperPrefix + NS_LITERAL_CSTRING("height_mm");
		rv = pref->GetIntPref(tmp.get(), &height);
		tmp = paperPrefix + NS_LITERAL_CSTRING("width_mm");
		rv = pref->GetIntPref(tmp.get(), &width);

		PRBool is_inch;
		tmp = paperPrefix + NS_LITERAL_CSTRING("is_inch");
		rv = pref->GetBoolPref(tmp.get(), &is_inch);

		nsCAutoString desc;
		if (is_inch)
		{
			double scale(25.4);
			desc = nsPrintfCString(20, _(" (%.1f x %.1f inch)"),
			                       width/scale, height/scale);
		}
		else
		{
			desc = nsPrintfCString(20, _(" (%d x %d mm)"), width, height);
		}
		tmp = name + desc;
		g_ptr_array_add(info->paper_desc_array, g_strdup(tmp.get()));
	}

	*aPrintInfo = info;

	return NS_OK;
}

NS_METHOD MozillaEmbedPrintInfoToPrintSettings(const EmbedPrintInfo *info,
                                               nsIPrintSettings **aPrintSettings)
{
	nsresult rv = NS_OK;

	nsCOMPtr<nsIPrintSettingsService> pss =
		do_GetService("@mozilla.org/gfx/printsettings-service;1", &rv);
	NS_ENSURE_SUCCESS(rv, rv);

	nsCOMPtr<nsIPrintSettings> options;
	rv = pss->GetNewPrintSettings(getter_AddRefs(options));
	NS_ENSURE_SUCCESS(rv, rv);

	rv = pss->InitPrintSettingsFromPrinter(NS_ConvertUTF8toUCS2(info->name).get(),
	                                       options);
	NS_ENSURE_SUCCESS(rv, rv);

	// Name field isn't inited. How dumb can we get?
	rv = options->SetPrinterName(NS_ConvertUTF8toUCS2(info->name).get());
	NS_ENSURE_SUCCESS(rv, rv);

	const static int frame_types[] = {
                nsIPrintSettings::kFramesAsIs,
                nsIPrintSettings::kSelectedFrame,
                nsIPrintSettings::kEachFrameSep
        };

        switch (info->pages)
        {
        case 0:
                break;
        case 1:
                options->SetPrintRange (nsIPrintSettings::kRangeSpecifiedPageRange);
                options->SetStartPageRange (info->from_page);
                options->SetEndPageRange (info->to_page);
                break;
        case 2:
                options->SetPrintRange (nsIPrintSettings::kRangeSelection);
                break;
        }

        options->SetMarginTop (info->top_margin);
        options->SetMarginBottom (info->bottom_margin);
        options->SetMarginLeft (info->left_margin);
        options->SetMarginRight (info->right_margin);

        options->SetHeaderStrLeft(NS_ConvertUTF8toUCS2(info->header_left_string).get());

        options->SetHeaderStrCenter(NS_ConvertUTF8toUCS2(info->header_center_string).get());

        options->SetHeaderStrRight(NS_ConvertUTF8toUCS2(info->header_right_string).get());

        options->SetFooterStrLeft(NS_ConvertUTF8toUCS2(info->footer_left_string).get());

        options->SetFooterStrCenter(NS_ConvertUTF8toUCS2(info->footer_center_string).get());

        options->SetFooterStrRight(NS_ConvertUTF8toUCS2(info->footer_right_string).get());

        options->SetToFileName (NS_ConvertUTF8toUCS2(info->file).get());

	options->SetPrintCommand (NS_ConvertUTF8toUCS2(info->command).get());

	/**
	 * Work around a mozilla bug where paper size & orientation are ignored
	 * and the specified file is created (containing invalid postscript)
	 * in print preview mode if we set "print to file" to true.
	 * See epiphany bug #119818.
	 */
	if (info->preview)
	{
		options->SetPrintToFile (PR_FALSE);
	}
	else
	{
		options->SetPrintToFile (info->print_to_file);
	}

	/* native paper size formats. Our dialog does not support custom yet */
	options->SetPaperSize (nsIPrintSettings::kPaperSizeNativeData);

	gchar *paperName = (gchar *)g_ptr_array_index(info->paper_array, info->paper);
	options->SetPaperName(NS_ConvertUTF8toUCS2(paperName).get());

        options->SetPrintInColor (info->print_color);
        options->SetOrientation (info->orientation);
        options->SetPrintFrameType (frame_types[info->frame_type]);

	NS_ADDREF(*aPrintSettings = options);

	return NS_OK;
}
